# Created for BADS 2018
# See README.md for details
# Python 3

"""
This module implements the directed graph data structure described in 
Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne. For more
information, see chapter 4.2 of the book.
"""

import sys

from itu.algs4.fundamentals.bag import Bag
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.graph import Graph
from itu.algs4.stdlib.instream import InStream


class Digraph:
    """
    The Graph class represents an undirected graph of vertices
    named 0 through V - 1.
    It supports the following two primary operations: add an edge to the graph,
    iterate over all of the vertices adjacent to a vertex. It also provides
    methods for returning the number of vertices V and the number
    of edges E. Parallel edges and self-loops are permitted.
    By convention, a self-loop v-v appears in the
    adjacency list of v twice and contributes two to the degree
    of v.

    This implementation uses an adjacency-lists representation, which 
    is a vertex-indexed array of Bag objects.
    All operations take constant time (in the worst case) except
    iterating over the vertices adjacent to a given vertex, which takes
    time proportional to the number of such vertices.
    """
    def __init__(self, V):
        """
        Initializes an empty graph with V vertices and 0 edges.
        param V the number of vertices

        :param V: number of vertices
        :raises: ValueError if V < 0
        """
        if V < 0: raise ValueError("Number of vertices must be nonnegative")
        self._V = V     # number of vertices
        self._E = 0     # number of edges
        self._adj = []  # adjacency lists

        for _ in range(V):
            self._adj.append(Bag()) # Initialize all lists to empty bags.

    @staticmethod
    def from_stream(stream):
        """
        Initializes a graph from the specified input stream.
        The format is the number of vertices V,
        followed by the number of edges E,
        followed by E pairs of vertices, with each entry separated by whitespace.
      
        :param stream: the input stream
        :returns: new graph from stream
        :raises ValueError: if the endpoints of any edge are not in prescribed range
        :raises ValueError: if the number of vertices or edges is negative
        :raises ValueError: if the input stream is in the wrong format 
        """
        V = stream.readInt()        # read V
        if V < 0: raise ValueError("Number of vertices must be nonnegative")
        g = Digraph(V)                # construct this graph
        E = stream.readInt()        # read E    
        if E < 0: raise ValueError("Number of edges in a Graph must be nonnegative")
        for _ in range(E):
            # Add an edge
            v = stream.readInt()    # read a vertex,
            w = stream.readInt()        # read another vertex,
            g._validateVertex(v)
            g._validateVertex(w)
            g.add_edge(v, w)             # and add edge connecting them.
        return g

    @staticmethod
    def from_graph(G):
        """
        Initializes a new graph that is a deep copy of G

        :param G: the graph to copy
        :returns: copy of G 
        """
        g = Graph(G.V())
        g._E = G.E()
        for v in range(G.V()):
            # reverse so that adjacency list is in same order as original
            reverse = Stack()
            for w in G._adj[v]:
                reverse.push(w)
            for w in reverse:
                g._adj[v].add(w)

    def V(self):
        """
        Returns the number of vertices in this graph.

        :returns: the number of vertices in this graph.
        """
        return self._V

    def E(self):
        """
        Returns the number of edges in this graph.

        :returns: the number of edges in this graph.
        """
        return self._E

    def _validateVertex(self, v):
        # throw a ValueError unless 0 <= v < V
        if v < 0 or v >= self._V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, self._V))

    def add_edge(self, v, w):
        """
        Adds the undirected edge v-w to this graph.

        :param v: one vertex in the edge
        :param w: the other vertex in the edge
        :raises ValueError: unless both 0 <= v < V and 0 <= w < V
        """
        self._adj[v].add(w)         # add w to v's list
        self._E += 1

    def adj(self, v):
        """
        Returns the vertices adjacent to vertex v.
     
        :param v: the vertex
        :returns: the vertices adjacent to vertex v, as an iterable
        :raises ValueError: unless  0 <= v < V
        """
        self._validateVertex(v)        
        return self._adj[v]

    def degree(self, v):
        """
        Returns the degree of vertex v.
     
        :param v: the vertex
        :returns: the degree of vertex v
        :raises ValueError:  unless 0 <= v < V
        """
        self._validateVertex(v)
        return self._adj[v].size()
    
    def reverse(self):
        """
        Returns the reverse of the digraph.
        
        :returns: the reverse of the digraph
        """
        rev = Digraph(self._V)
        for v in range(self._V):
            for w in self.adj(v):
                rev.add_edge(w, v)
        return rev

    def __repr__(self):
        """
        Returns a string representation of this graph.
     
        :returns: the number of vertices V, followed by the number of edges E,
                    followed by the V adjacency lists
        """
        s = ["{} vertices, {} edges\n".format(self._V, self._E)]
        for v in range(self._V):
            s.append("%d : " % (v))
            for w in self._adj[v]:
                s.append("%d " % (w))
            s.append("\n")

        return ''.join(s)
    
    

if __name__ == '__main__':
    # Create stream from file or the standard input,
    # depending on whether a file name was passed.
    stream = sys.argv[1] if len(sys.argv) > 1 else None
    
    d = Digraph.from_stream(InStream(stream))
    print(d)



import sys

from itu.algs4.errors.errors import IllegalArgumentException
from itu.algs4.fundamentals.bag import Bag
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.directed_edge import DirectedEdge
from itu.algs4.stdlib.instream import InStream

# Created for BADS 2018
# See README.md for details
# Python 3


class EdgeWeightedDigraph:
    """
    The EdgeWeightedDigraph class represents an edge-weighted
    digraph of vertices named 0 through V-1, where each
    directed edge is of type DirectedEdge and has a real-valued weight.
    It supports the following two primary operations: add a directed edge
    to the digraph and iterate over all edges incident from a given vertex.
    it also provides methods for returning the number of vertices V
    and the number of edges E. Parallel edges and self-loops are permitted.
    This implementation uses an adjacency-lists representation, which
    is a vertex-indexed array of Bag objects.
    All operations take constant time (in the worst case) except
    iterating over the edges incident from a given vertex, which takes
    time proportional to the number of such edges.
    """
    def __init__(self, V):
        """
        Initializes an empty edge-weighted digraph with V vertices and 0 edges.
        :param V: the number of vertices
        :raises IllegalArgumentException: if V < 0
        """
        if V < 0:
            raise IllegalArgumentException("Number of vertices in a Digraph must be nonnegative")
        self._V = V
        self._E = 0
        self._indegree = [0] * V
        self._adj = [None] * V
        for v in range(V):
            self._adj[v] = Bag()

    @staticmethod
    def from_graph(G):
        """
        Initializes a new edge-weighted digraph that is a deep copy of G.
        :param G: the edge-weighted digraph to copy
        :return: a copy of graph G
        :rtype: EdgeWeightedDigraph
        """
        g = EdgeWeightedDigraph(G.V())
        g._E = G.E()
        for v in range(G.V()):
            g._indegree[v] = G.indegree(v)
            reverse = Stack()
            for e in G.adj(v):
                reverse.push(e)
            for e in reverse:
                g._adj[v].add(e)
        return g

    @staticmethod
    def from_stream(stream):
        """
        Initializes an edge-weighted digraph from the specified input stream.
        The format is the number of vertices V,
        followed by the number of edges E,
        followed by E pairs of vertices and edge weights,
        with each entry seperated by whitespace
        :param stream: the input stream

        :raises IllegalArgumentException: if the endpoints of any edge are not in prescribed range
        :raises IllegalArgumentException: if the number of vertices or edges is negative
        :return: the edge-weighted digraph
        :rtype: EdgeWeightedDigraph
        """
        g = EdgeWeightedDigraph(stream.readInt())
        E = stream.readInt()
        if g._E < 0:
            raise IllegalArgumentException("Number of edges must be nonnegative")
        for i in range(E):
            v = stream.readInt()
            w = stream.readInt()
            g._validate_vertex(v)
            g._validate_vertex(w)
            weight = stream.readFloat()
            g.add_edge(DirectedEdge(v, w, weight))
        return g

    def V(self):
        """
        Returns the number of vertices in this edge-weighted digraph.
        :return: the number of vertices in this edge-weighted digraph
        :rtype: int
        """
        return self._V

    def E(self):
        """
        Returns the number of edges in this edge-weighted digraph.
        :return: the number of edges in this edge-weighted digraph
        :rtype: int
        """
        return self._E

    def _validate_vertex(self, v):
        """
        Raises an IllegalArgumentException unluess 0 <= v < V
        :param v: the vertex to validate
        """
        if v < 0 or v >= self._V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v, self._V-1))

    def add_edge(self, e):
        """
        Adds the directed edge e to this edge-weighted digraph.
        :param e: the edge
        :raises IllegalArgumentException: unless endpoints of edge are between 0 and V-1
        """
        v = e.from_vertex()
        w = e.to_vertex()
        self._validate_vertex(v)
        self._validate_vertex(w)
        self._adj[v].add(e)
        self._indegree[w] += 1
        self._E += 1

    def adj(self, v):
        """
        Returns the directed edges incident from vertex v.
        :param v: the vertex
        :return: the directed edges incident from vertex v.
        :rtype: collections.iterable[DirectedEdge]
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._adj[v]

    def outdegree(self, v):
        """
        Returns the number of directed edges incident from vertex v.
        This is known as the outdegree of vertex v.
        :param v: the vertex
        :return: the outdegree of vertex v
        :rtype: int
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._adj[v].size()

    def indegree(self, v):
        """
        Returns the number of directed edges incident to vertex v.
        This is known as the indegree of vertex v.
        :param v: the vertex
        :return: the indegree of vertex v
        :rtype: int
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._indegree[v]

    def edges(self):
        """
        Returns all directed edges in this edge-weighted digraph.
        :return: all edges in this edge-weighted digraph
        :rtype: collections.iterable[DirectedEdge]
        """
        edges = Bag()
        for v in range(self._V):
            for e in self._adj[v]:
                edges.add(e)
        return edges

    def __repr__(self):
        """
        Returns a string representation of this edge-weighted digraph.
        :return: the number of vertices V, followed by the number of edges E,
        followed by the V adjacency lists of edges.
        :rtype: str
        """
        s = ["{} {} \n".format(self._V, self._E)]
        for v in range(self._V):
            s.append("{}: ".format(v))
            for e in self._adj[v]:
                s.append("{}  ".format(e))
            s.append("\n")
        return ''.join(s)


def main():
    """
    Creates an edge-weighted digraph from the given input file and prints it.
    """
    if len(sys.argv) > 1:
        stream = InStream(sys.argv[1])
        G = EdgeWeightedDigraph.from_stream(stream)
        print(G)


if __name__ == '__main__':
    main()



import sys

from itu.algs4.stdlib import stdio

# Created for BADS 2018
# see README.md for details
# Python 3

"""
The selection module provides a function for sorting an
array using selection sort.
"""


def sort(a):
    """
    Rearranges the array in ascending order, using the natural order.
    :param a: the array to be sorted
    """
    _n = len(a)
    for i in range(_n):
        _min = i
        for j in range(i+1, _n):
            if a[j] < a[_min]:
                _min = j
        _exch(a, i, _min)


def _exch(a, i, j):
    """
    Exchanges the the items at index i and j.
    :param a: the array to be sorted
    :param i: the index for the first item
    :param j: the index for the second item
    """
    a[i], a[j] = a[j], a[i]


def _show(a):
    """
    Prints the content of the array.
    :param a: The array to be shown
    """
    for i in range(len(a)):
        print(a[i])


def main():
    """
    Reads strings from stdin, sorts them, and prints the result to stdout.
    """
    a = stdio.readAllStrings()
    sort(a)
    _show(a)


if __name__ == '__main__':
    main()



# Created for BADS 2018
# See README.md for details
# Python 3

"""
This module implements the directed cycle algorithm for EdgeWeightedDigraphs described in 
Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne. This version
works for both weighted and unweighted directed graphs, due to Python's duck-typing.

For more
information, see chapter 4.2 of the book.
"""

import sys

from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.stdlib import instream


class EdgeWeightedDirectedCycle:
    """
    The EdgeWeightedDirectedCycle class represents a data type for determining whether 
    edge-weighted digraph has a directed cycle. The hasCycle operation determines whether 
    the edge-weighted digraph has a directed cycle and, if so, the cycle operation returns one.
    
    This implementation uses depth-first search. The constructor takes time proportional to 
    V + E (in the worst case), where V is the number of vertices and E is the number of edges. 
    Afterwards, the hasCycle operation takes constant time; the cycle operation takes time 
    proportional to the length of the cycle.

    See Topological to compute a topological order if the edge-weighted digraph is acyclic.

    For additional documentation, see Section 4.4 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """    
    
    def __init__(self, edge_weighted_digraph):
        """
        Determines whether the edge weighted digraph has a directed cycle and, if so,
        finds such a cycle.
        
        :digraph: the digraph
        """
        self._cycle = None
        self._on_stack = [False]*edge_weighted_digraph.V()
        self._edge_to  = [None]*edge_weighted_digraph.V()
        self._marked = [False]*edge_weighted_digraph.V()
        for v in range(edge_weighted_digraph.V()):
            if not self._marked[v]:
                self._dfs(edge_weighted_digraph, v) 
    
    # check that algorithm computes either the topological order or finds a directed cycle
    def _dfs(self, graph, v):
        self._on_stack[v] = True
        self._marked[v] = True
        for edge in graph.adj(v):
            w = edge.to_vertex()
            
            # short circuit if directed cycle found
            if self.has_cycle():
                return
            # found new vertex, so recur
            elif not self._marked[w]:
                self._edge_to[w] = edge
                self._dfs(graph, w)
            # trace back directed cycle
            elif self._on_stack[w]:
                self._cycle = Stack()
                f = edge
                while f.from_vertex() != w:
                    self._cycle.push(f)
                    f = self._edge_to[f.from_vertex()]
                self._cycle.push(f)
                
        self._on_stack[v] = False
    
    def has_cycle(self):
        """
        Does the edge weighted digraph have a directed cycle?
        
        :returns: true if there is a cycle, false otherwise
        """
        return self._cycle != None
    
    def cycle(self):
        """
        Returns a directed cycle if the edge weighted digraph has a directed cycle, and null otherwise.
        
        :returns: a directed cycle (as an iterable) if the digraph has a directed cycle, and null otherwise
        """
        return self._cycle



if __name__ == '__main__':
    # Create stream from file or the standard input,
    # depending on whether a file name was passed.
    stream = sys.argv[1] if len(sys.argv) > 1 else None
    
    d = EdgeWeightedDigraph.from_stream(instream.InStream(stream))
 
    cyc = EdgeWeightedDirectedCycle(d)
    print(cyc.cycle())



# Created for BADS 2018
# See README.md for details
# Python 3

"""
This module provides functions for sorting an array using bottom-up mergesort.


For additional documentation, see Section 2.1 of Algorithms, 4th Edition 
by Robert Sedgewick and Kevin Wayne.
"""
#  Sorts a sequence of strings from standard input using mergesort

def _is_sorted(a, lo=0, hi=None):
    # If hi is not specified, use whole array
    if hi is None:
        hi = len(a)
        
    # check if sublist is sorted
    for i in range(lo+1, hi):
        if a[i] < a[i-1]:
            return False
    return True

# stably merge a[lo .. mid] with a[mid+1 ..hi] using aux[lo .. hi]
def _merge(a, aux, lo, mid, hi):
    # copy to aux[]
    for k in range(lo, hi+1):
        aux[k] = a[k]
        
    # merge back to a[]
    i, j = lo, mid+1
    for k in range(lo, hi+1):
        if i > mid:
            a[k] = aux[j]
            j += 1
        elif j > hi:
            a[k] = aux[i]
            i += 1
        elif aux[j] < aux[i]:
            a[k] = aux[j]
            j += 1
        else:
            a[k] = aux[i]
            i += 1

def sort(a):
    """
    Rearranges the array in ascending order, using the natural order.
    
    :param a: the array to be sorted
    """
    n = len(a)
    aux = [None]*n
    
    length = 1
    while length < n:
        lo = 0
        while lo < n-length:
            mid = lo+length-1
            hi = min(mid+length, n-1)
            _merge(a, aux, lo, mid, hi)
            lo += 2*length
        length *= 2
    
    assert _is_sorted(a)
    

import sys

from itu.algs4.stdlib import stdio

# Reads in a sequence of strings from standard input or a file 
# supplied as argument to the program; mergesorts them; 
# and prints them to standard output in ascending order.
if __name__ == '__main__':
    if len(sys.argv) > 1:
        try: 
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
    
    a = stdio.readAllStrings()
    sort(a)
    for elem in a:
        print(elem)



# Created for BADS 2018
# See README.md for details
# Python 3

"""
 *  Execution:    python kosaraju_sharir_scc.py filename.txt
 *  Dependencies: Digraph TransitiveClosure InStream DepthFirstOrder
 *  Data files:   https:#algs4.cs.princeton.edu/42digraph/tinyDG.txt
 *                https:#algs4.cs.princeton.edu/42digraph/mediumDG.txt
 *                https:#algs4.cs.princeton.edu/42digraph/largeDG.txt
 *
 *  Compute the strongly-connected components of a digraph using the
 *  Kosaraju-Sharir algorithm.
 *
 *  Runs in O(E + V) time.
 *
 *  % python kosaraju_sharir_scc.py tinyDG.txt
 *  5 strong components
 *  1 
 *  0 2 3 4 5 
 *  9 10 11 12 
 *  6 8 
 *  7
 *
"""
import sys

from itu.algs4.errors.errors import IllegalArgumentException
from itu.algs4.fundamentals.queue import Queue
from itu.algs4.graphs.depth_first_order import DepthFirstOrder
from itu.algs4.graphs.digraph import Digraph
from itu.algs4.graphs.transitive_closure import TransitiveClosure
from itu.algs4.stdlib.instream import InStream


class KosarajuSharirSCC:

    """
     * Computes the strong components of the digraph G.
     * @param G the digraph
     """
    def __init__(self, G):
        self._marked = [False]*G.V()     # marked[v] = has vertex v been visited?
        self._id = [0]*G.V()             # id[v] = id of strong component containing v
        self._count = 0                  # number of strongly-connected components

        # compute reverse postorder of reverse graph
        dfo = DepthFirstOrder(G.reverse())

        # run DFS on G, using reverse postorder to guide calculation
        for v in dfo.reverse_post():
            if not self._marked[v]:
                self._dfs(G, v)
                self._count+=1

        # check that id[] gives strong components
        assert self._check(G)
    

    # DFS on graph G
    def _dfs(self, G, v): 
        self._marked[v] = True
        self._id[v] = self._count
        for w in G.adj(v):
            if not self._marked[w]:
                self._dfs(G, w)
        
    """
     * Returns the number of strong components.
     * @return the number of strong components
     """
    def count(self):
        return self._count

    """
     * Are vertices v and w in the same strong component?
     * @param  v one vertex
     * @param  w the other vertex
     * @return true if vertices v and w are in the same
     *         strong component, and false otherwise
     * @throws IllegalArgumentException unless 0 <= v < V
     * @throws IllegalArgumentException unless 0 <= w < V
     """
    def strongly_connected(self, v, w):
        self._validate_vertex(v)
        self._validate_vertex(w)
        return self._id[v] == self._id[w]
    

    """
     * Returns the component id of the strong component containing vertex v.
     * @param  v the vertex
     * @return the component id of the strong component containing vertex v
     * @throws IllegalArgumentException unless 0 <= s < V
     """
    def id(self, v):
        self._validate_vertex(v)
        return self._id[v]

    # does the id[] array contain the strongly connected components?
    def _check(self, G):
        tc =  TransitiveClosure(G)
        for v in range(G.V()): 
            for w in range(G.V()):
                if self.strongly_connected(v, w) != (tc.reachable(v, w) and tc.reachable(w, v)):
                    return False
        return True
    

    # throw an IllegalArgumentException unless 0 <= v < V
    def _validate_vertex(self, v):
        V = len(self._marked)
        if v < 0 or v >= V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v,V-1))
    

def main(args):
    stream =  InStream(args[0])
    G =  Digraph.from_stream(stream)
    scc =  KosarajuSharirSCC(G)

    # number of connected components
    m = scc.count()
    print("{} strong components".format(m))

    # compute list of vertices in each strong component
    components = [Queue() for i in range(m)]
    
    for v in range(G.V()):
        components[scc.id(v)].enqueue(v)
    
    # print results
    for i in range(m):
        for v in components[i]: 
            print(str(v), end=' ')
        print()

if __name__ == '__main__':
    main(sys.argv[1:])



# Created for BADS 2018
# see README.md for details
# This is python3 

import math

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.stdlib import stdio


class BreadthFirstPaths:
    """
    The BreadthFirstPaths class represents a data type for finding
    shortest paths (number of edges) from a source vertex s
    (or a set of source vertices)
    to every other vertex in a directed or undirected graph.
    
    This implementation uses breadth-first search.
    The constructor takes time proportional to V + E,
    where V is the number of vertices and E is the number of edges.
    Each call to distTo(int) and hasPathTo(int) takes constant time
    each call to pathTo(int) takes time proportional to the length
    of the path.
    It uses extra space (not including the graph) proportional to V.
    """


    def __init__(self, G, s):
        """
        Computes the shortest path between the source vertex s
        and every other vertex in the graph G.

        :param G: the graph
        :param s: the source vertex
        :raises ValueError: unless 0 <= s < V
        """
        self._marked = [False] * G.V()      # Is a shortest path to this vertex known?
        self._dist_to = [math.inf] * G.V()
        self._edgeTo = [0] * G.V()          # last vertex on known path to this vertex        
        self._validateVertex(s)
        self._bfs(G, s)

        assert self._check(G, s)

    # @staticmethod
    # def from_multiple_sources(G, sources):
    #     """
    #     Computes the shortest path between any one of the source vertices in sources
    #     and every other vertex in graph G.

    #     :param G the graph
    #     :param sources the source vertices
    #     :raises ValueError: unless 0 <= s < V for each vertex
    #                         s in sources
    #     """
    #     pass

    def _bfs(self, G, s):
        # breadth-first search from a single source
        queue = Queue()
        self._dist_to[s] = 0
        self._marked[s] = True              # Mark the source
        queue.enqueue(s)                    #   and put it on the queue.

        while not queue.is_empty():
            v = queue.dequeue()             # Remove next vertex from the queue.
            for w in G.adj(v):
                if not self._marked[w]:
                    self._edgeTo[w] = v     # For every unmarked adjacent vertex,
                    self._dist_to[w] = self._dist_to[v] + 1
                    self._marked[w] = True  # mark it because path is known,
                    queue.enqueue(w)        # and add it to the queue.

    # def _bfs_multiple_sources(self, G, sources):
    #     # breadth-first search from multiple sources
    #     pass

    def has_path_to(self, v):
        """
        Is there a path between the source vertex s (or sources) and vertex v?

        :param v: the vertex
        :returns: true if there is a path, and False otherwise
        :raises ValueError: unless 0 <= v < V
        """
        return self._marked[v]

    def dist_to(self, v):
        """
        Returns the number of edges in a shortest path between the source vertex s
        (or sources) and vertex v?

        :param v: the vertex
        :returns: the number of edges in a shortest path
        :raises ValueError: unless 0 <= v < V
        """
        self._validateVertex(v)
        return self._dist_to[v]

    def path_to(self, v):
        """
        Returns a shortest path between the source vertex s (or sources)
        and v, or null if no such path.

        :param v: the vertex
        :returns: the sequence of vertices on a shortest path, as an Iterable
        :raises ValueError: unless 0 <= v < V
        """
        if not self.has_path_to(v): return None
        path = Stack()
        x = v
        while self._dist_to[x] != 0:
            path.push(x)
            x = self._edgeTo[x]
        path.push(x)
        return path

    def _check(self, G, s):
        # check optimality conditions for singe source
        # check that the distance of s = 0
        if (self._dist_to[s] != 0):
            stdio.writef("distance of source %i to itself = %i\n", s, self._dist_to[s])
            return False

        # check that for each edge v-w dist[w] <= dist[v] + 1
        # provided v is reachable from s
        for v in range(G.V()):
            for w in G.adj(v):
                #if self.has_path_to(v) != self.has_path_to(w):
                # modified for directed graphs
                if self.has_path_to(v) and not self.has_path_to(w):
                    stdio.writef("edge %i-%i\n", v, w)
                    stdio.writef("has_path_to(%i) = %s\n", v, self.has_path_to(v))
                    stdio.writef("has_path_to(%i) = %s\n", w, self.has_path_to(w))
                    return False
                if self.has_path_to(v) and (self._dist_to[w] > self._dist_to[v] + 1):
                    stdio.writef("edge %i-%i\n", v, w)
                    stdio.writef("dist_to[%i] = %i\n", v, self._dist_to[v])
                    stdio.writef("dist_to[%i] = %i\n", v, self._dist_to[w])
                    return False

        # check that v = edgeTo[w] satisfies distTo[w] = distTo[v] + 1
        # provided v is reachable from s
        for w in range(G.V()):
            if not self.has_path_to(w) or w == s: continue
            v = self._edgeTo[w]
            if self._dist_to[w] != self._dist_to[v] + 1:
                stdio.writef("shortest path edge %i-%i\n", v, w)
                stdio.writef("dist_to[%i] = %i\n", v, self._dist_to[v])
                stdio.writef("dist_to[%i] = %i\n", w, self._dist_to[w])
                return False

        return True

    def _validateVertex(self, v):
        # throw an ValueError unless 0 <= v < V
        V = len(self._marked)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))

    # def _validateVertices(self, vertices):
    #     # throw an ValueError unless 0 <= v < V
    #     pass

class BreadthFirstPathsBook:
    def __init__(self, G, s):
        self._marked = [False] * G.V()  # Is a shortest path to this vertex known?
        self._edgeTo = [0] * G.V()      # last vertex on known path to this vertex
        self._s = s                     # source
        self._bfs(G, s)

    def _bfs(self, G, s):
        # breadth-first search from a single source
        queue = Queue()
        self._marked[s] = True              # Mark the source
        queue.enqueue(s)                    #   and put it on the queue.
        while not queue.is_empty():
            v = queue.dequeue()             # Remove next vertex from the queue.
            for w in G.adj(v):
                if not self._marked[w]:
                    self._edgeTo[w] = v     # For every unmarked adjacent vertex,
                    self._marked[w] = True  # mark it because path is known,
                    queue.enqueue(w)        # and add it to the queue.

    def has_path_to(self, v):
        return self._marked[v]

    def path_to(self, v):
        if not self.has_path_to(v): return None
        path = Stack()
        x = v
        while x != self._s:
            path.push(x)
            x = self._edgeTo[x]
        path.push(self._s)
        return path

if __name__ == "__main__":
    import sys
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.graph import Graph
    from itu.algs4.stdlib.instream import InStream

    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    s = int(sys.argv[2])
    bfs  = BreadthFirstPaths(G, s)

    for v in range(G.V()):
        if bfs.has_path_to(v):
            stdio.writef("%d to %d (%d):  ", s, v, bfs.dist_to(v))
            for x in bfs.path_to(v):
                if x == s: stdio.write(x)
                else:      stdio.writef("-%i", x)            
            stdio.writeln()
        else:
            stdio.writef("%d to %d (-):  not connected\n", s, v)



# Created for BADS 2018
# See README.md for details
# Python 3
import sys

from itu.algs4.stdlib import stdio

# data files:
# https://algs4.cs.princeton.edu/35applications/amino.csv
#  Data files:   https://algs4.cs.princeton.edu/35applications/DJIA.csv
#                https://algs4.cs.princeton.edu/35applications/UPC.csv
#                https://algs4.cs.princeton.edu/35applications/amino.csv
#                https://algs4.cs.princeton.edu/35applications/elements.csv
#                https://algs4.cs.princeton.edu/35applications/ip.csv
#                https://algs4.cs.princeton.edu/35applications/morse.csv
 

#  Execution:    python lookup_csv.py file.csv keyField valField
#  
#  Reads in a set of key-value pairs from a two-column CSV file
#  specified on the command line; then, reads in keys from standard
#  input and prints out corresponding values.
# 
#  % python lookup_csv.py amino.csv 0 3     % python lookup_csv.py ip.csv 0 1 
#  TTA                                  www.google.com 
#  Leucine                              216.239.41.99 
#  ABC                               
#  Not found                            % python lookup_csv.py ip.csv 1 0 
#  TCT                                  216.239.41.99 
#  Serine                               www.google.com 
#                                 
#  % python lookup_csv.py amino.csv 3 0     % python lookup_csv.py DJIA.csv 0 1 
#  Glycine                              29-Oct-29 
#  GGG                                  252.38 
#                                       20-Oct-87 
#                                       1738.74

"""
The LookupCSV class provides a data-driven client for reading in a
key-value pairs from a file; then, printing the values corresponding to the
keys found on standard input. Both keys and values are strings.
The fields to serve as the key and value are taken as command-line arguments.
"""

if __name__ == '__main__':
  args = sys.argv[1:]
  keyField = int(args[1])
  valField = int(args[2])

  # symbol table
  st = {}

  # read in the data from csv file
  file = open(args[0], 'r')
  line = file.readline()
  while line:
      tokens = line.split(",")
      key = tokens[keyField]
      val = tokens[valField]
      st[key] = val        
      line = file.readline()

  while not stdio.isEmpty():
      s = stdio.readString()
      if s in st:   
          print(st.get(s))
      else: 
          print("Not found") 
  file.close()



# Created for BADS 2018
# See README.md for details
# Python 3


class NoSuchElementException(Exception):
    pass


class IllegalArgumentException(Exception):
    pass


class UnsupportedOperationException(Exception):
    pass



import math

from itu.algs4.errors.errors import IllegalArgumentException

# Created for BADS 2018
# See README.md for details
# Python 3


class DirectedEdge:
    """
    The DirectedEdge class represents a weighted edge in an
    EdgeWeightedDigraph. Each edge consists of two integers
    (naming the two vertices) and a real-value weight. The data type
    provides methods for accessing the two endpoints of the directed edge and
    the weight.
    """
    def __init__(self, v, w, weight):
        """
        Initializes a directed edge from vertex v to vertex w with
        the given weight.
        :param v: the tail vertex
        :param w: the head vertex
        :param weight: the weight of the directed edge
        :raises IllegalArgumentException: if either v or w is a negative integer
        :raises IllegalArgumentException: if weight is NaN
        """
        if v < 0:
            raise IllegalArgumentException("Vertex names must be nonnegative integers")
        if w < 0:
            raise IllegalArgumentException("Vertex names must be nonnegative integers")
        if math.isnan(weight):
            raise IllegalArgumentException("Weight is NaN")
        self._v = v
        self._w = w
        self._weight = weight

    def from_vertex(self):
        """
        Returns the tail vertex of the directed edge.
        :return: the tail vertex of the directed edge
        :rtype: int
        """
        return self._v

    def to_vertex(self):
        """
        Returns the head vertex of the directed edge.
        :return: the head vertex of the directed edge
        :rtype: int
        """
        return self._w

    def weight(self):
        """
        Returns the weight of the directed edge.
        :return: the weight of the directed edge
        :rtype: float
        """
        return self._weight

    def __repr__(self):
        """
        Returns a string representation of the directed edge.
        :return: a string representation of the directed edge
        :rtype: str
        """
        return "{}->{} {:5.2f}".format(self._v, self._w, self._weight)


def main():
    """
    Creates a directed edge and prints it.
    :return:
    """
    e = DirectedEdge(12, 34, 5.67)
    print(e)


if __name__ == '__main__':
    main()



from setuptools import find_packages, setup

setup(
    name="itu.algs4",
    version="0.2.5",
    description='Python 3 port of the Java code in "Algorithms, 4th Edition" by Sedgewick and Wayne',
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author='Algorithms group at ITU Copenhagen',
    # author_email='',
    url="https://github.com/itu-algorithms/itu.algs4/",
    license="GNU General Public License v3 (GPLv3)",
    packages=find_packages(exclude=["examples", "tests"]),
    include_package_data=True,
    extras_require={
        "audio": ["numpy"],
        "visual": ["pygame"],
        "dev": ["flake8", "black", "isort", "pytest", "pytest-cov", "coveralls", "mypy"],
    },
    zip_safe=False,
    platforms="any",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
outstream.py

The outstream module defines the OutStream class.
"""

import sys

#-----------------------------------------------------------------------

class OutStream:

    """
    An OutStream object wraps around a text file or sys.stdout, and
    supports writing to that stream.
    """

    #-------------------------------------------------------------------

    def __init__(self, f=None):
        """
        Construct self to wrap around a stream. If f is provided, then
        the stream is a file of that name.  Otherwise the stream is
        standard output.
        """
        if f is None:
            self._stream = sys.stdout
        else:
            if sys.hexversion < 0x03000000:
                self._stream = open(f, 'w')
            else:
                self._stream = open(f, 'w', encoding='utf-8')

    #-------------------------------------------------------------------

    def writeln(self, x=''):
        """
        Write x and an end-of-line mark to the stream wrapped by self.
        """
        if sys.hexversion < 0x03000000:
            x = unicode(x)
            x = x.encode('utf-8')
        else:
            x = str(x)
        self._stream.write(x)
        self._stream.write('\n')
        self._stream.flush()

    #-------------------------------------------------------------------

    def write(self, x=''):
        """
        Write x to the stream wrapped by self.
        """
        if (sys.hexversion < 0x03000000):
            x = unicode(x)
            x = x.encode('utf-8')
        else:
            x = str(x)
        self._stream.write(x)
        self._stream.flush()

    #-------------------------------------------------------------------

    def writef(self, fmt, *args):
        """
        Write each element of args to the stream wrapped by self.
        Use the format specified by string fmt.
        """
        x = fmt % args
        if sys.hexversion < 0x03000000:
            x = unicode(x)
            x = x.encode('utf-8')
        self._stream.write(x)
        self._stream.flush()

    #-------------------------------------------------------------------

    def __del__(self):
        """
        Close the stream wrapped by self.
        """
        self._stream.close()



from typing import Generic, Iterator, Optional, TypeVar

from ..errors.errors import NoSuchElementException

# Created for BADS 2018
# See README.md for details
# This is python3


T = TypeVar('T')

class Node(Generic[T]):
    def __init__(self, item: T, next: Optional['Node[T]']) -> None:
        """
        Initializes a new node
        :param item: the item to be stored in the node
        :param next: the next node in the queue
        """
        self.item: T = item
        self.next: Optional[Node[T]] = next

class Queue(Generic[T]):
    """
    The Queue class represents a first-in-first-out (FIFO)
    queue of generic items.
    It supports the usual enqueue and dequeue
    operations, along with methods for peeking at the first item,
    testing if the queue is empty, and iterating through the items in FIFO order
    This implementation uses a singly linked list of linked-list nodes
    The enqueue, dequeue, peek, size, and is_empty operations all take constant time in the worst case
    """

    def __init__(self) -> None:
        """
        Initializes an empty queue.
        """
        self._first: Optional[Node[T]] = None
        self._last : Optional[Node[T]] = None
        self._n: int = 0

    def enqueue(self, item: T) -> None:
        """
        Adds the item to this queue.
        :param item: the item to add
        """
        old_last: Optional[Node[T]] = self._last
        self._last = Node(item, None)
        if self.is_empty():
            self._first = self._last
        else:
            assert old_last is not None
            old_last.next = self._last
        self._n += 1

    def dequeue(self) -> T:
        """
        Removes and returns the item on this queue that was least recently added.
        :return: the item on this queue that was least recently added.
        :raises NoSuchElementException: if this queue is empty
        """
        if self.is_empty():
            raise NoSuchElementException("Queue underflow")

        assert self._first is not None
        item = self._first.item
        self._first = self._first.next
        self._n -= 1
        if self.is_empty():
            self._last = None
        return item

    def is_empty(self) -> bool:
        """
        Returns true if this queue is empty.
        :return: True if this queue is empty otherwise False
        :rtype: bool
        """
        return self._first is None

    def size(self) -> int:
        """
        Returns the number of items in this queue.
        :return: the number of items in this queue
        :rtype: int
        """
        return self._n

    def __len__(self) -> int:
        return self.size()

    def peek(self) -> T:
        """
        Returns the item least recently added to this queue.
        :return: the item least recently added to this queue
        :raises NoSuchElementException: if this queue is empty
        """
        if self.is_empty():
            raise NoSuchElementException("Queue underflow")

        assert self._first is not None
        return self._first.item

    def __iter__(self) -> Iterator[T]:
        """
        Iterates over all the items in this queue in FIFO order.
        """
        curr = self._first
        while curr is not None:
            yield curr.item
            curr = curr.next

    def __repr__(self) -> str:
        """
        Returns a string representation of this queue.
        :return: the sequence of items in FIFO order, separated by spaces
        """
        s = []
        for item in self:
            s.append("{} ".format(item))
        return ''.join(s)



import math

from itu.algs4.errors.errors import IllegalArgumentException

# Created for BADS 2018
# See README.md for details
# Python 3


class Edge:
    """
    The Edge class represents a weighted edge in an
    EdgeWeightedGraph. Each edge consists of two integers
    (naming the two vertices) and a real-value weight. The data type
    provides methods for accessing the two endpoints of the edge and
    the weight. The natural order for this data type is by
    ascending order of weight.
    """
    def __init__(self, v, w, weight):
        """
        Initializes an edge between vertices v and w of
        the given weight.
        :param v: one vertex
        :param w: the other vertex
        :param weight: the weight of this edge
        :raises IllegalArgumentException: if either v or w is a negative integer
        :raises IllegalArgumentException: if weight is NaN
        """
        if v < 0:
            raise IllegalArgumentException("vertex index must be a nonnegative integer")
        if w < 0:
            raise IllegalArgumentException("vertex index must be a nonnegative integer")
        if math.isnan(weight):
            raise IllegalArgumentException("Weight is NaN")
        self._v = v
        self._w = w
        self._weight = weight

    def weight(self):
        """
        Returns the weight of this edge.
        :return: the weight of this edge
        :rtype: float
        """
        return self._weight

    def either(self):
        """
        Returns either endpoint of this edge.
        :return: either endpoint of this edge
        :rtype: int
        """
        return self._v

    def other(self, vertex):
        """
        Returns the endpoint of this edge that is different from the given vertex.
        :param vertex: one endpoint of this edge
        :return: the other endpoint of this edge
        :rtype: int
        :raises IllegalArgumentException: if the vertex is not one of the endpoints of this edge
        """
        if vertex == self._v:
            return self._w
        elif vertex == self._w:
            return self._v
        else:
            raise IllegalArgumentException("Illegal endpoint")

    def __lt__(self, other):
        """
        Checks if this edge has smaller weight than other edge
        :param other: the edge to compare with
        :return: True if weight of this edge is less than weight of other edge otherwise returns False
        """
        return self.weight() < other.weight()

    def __gt__(self, other):
        """
        Checks if this edge has greater weight than other edge
        :param other: the edge to compare with
        :return: True if weight of this edge is greater than weight of other edge otherwise returns False
        """
        return self.weight() > other.weight()

    def __repr__(self):
        """
        Returns a string representation of this edge.
        :return: a string representation of this edge
        """
        return "{}-{} {:.5f}".format(self._v, self._w, self._weight)


def main():
    """
    Creates an edge and prints it.
    """
    e = Edge(12, 34, 5.67)
    print(e)


if __name__ == '__main__':
    main()



# Created for BADS 2018
# See README.md for details
# Python 3
import os
import sys

import itu.algs4.stdlib.stdrandom
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.directed_edge import DirectedEdge
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph

# Execution:    python edge_weighted_directed_cycle V E F
# Finds a directed cycle in an edge-weighted digraph.
# Runs in O(E + V) time.

"""
    The EdgeWeightedDirectedCycle class represents a data type for 
    determining whether an edge-weighted digraph has a directed cycle.
    The hasCycle operation determines whether the edge-weighted
    digraph has a directed cycle and, if so, the cycle operation
    returns one.
    This implementation uses depth-first search.
    The constructor takes time proportional to V + E
    (in the worst case),
    where V is the number of vertices and E is the number of edges.
    Afterwards, the hasCycle operation takes constant time;
    the cycle operation takes time proportional
    to the length of the cycle.
"""
class EdgeWeightedDirectedCycle:
    """
     Determines whether the edge-weighted digraph G has a directed cycle and,
     if so, finds such a cycle.
     :param G the edge-weighted digraph
    """
    def __init__(self, G):
        self._marked = [False] * G.V()       # marked[v] = has vertex v been marked?
        self._edgeTo = [None] * G.V()        # edgeTo[v] = previous DirectedEdge on path to v
        self._onStack = [False] * G.V()      # onStack[v] = is vertex on the stack?
        self._cycle = None                   # directed cycle (or None if no such cycle)

        for v in range(G.V()):
            if not self._marked[v]: 
                self._dfs(G, v)

        # check that digraph has a cycle
        assert self._check()

    # check that algorithm computes either the topological order or finds a directed cycle
    def _dfs(self, G, v):
        self._onStack[v] = True
        self._marked[v] = True
        for e in G.adj(v):
            w = e.to_vertex()

            # short circuit if directed cycle found
            if self._cycle is not None: 
                return

            # found new vertex, so recur
            elif not self._marked[w]:
                self._edgeTo[w] = e
                self._dfs(G, w)
            
            # trace back directed cycle
            elif self._onStack[w]:
                self._cycle = Stack()
                f = e
                while f.from_vertex() != w:
                    self._cycle.push(f)
                    f = self._edgeTo[f.from_vertex()]
                
                self._cycle.push(f)
                return
        self._onStack[v] = False


    # Does the edge-weighted digraph have a directed cycle?
    # @return True if the edge-weighted digraph has a directed cycle,
    # False otherwise
    def has_cycle(self):
        return self._cycle is not None

    # Returns a directed cycle if the edge-weighted digraph has a directed cycle,
    # and None otherwise.
    # @return a directed cycle (as an iterable) if the edge-weighted digraph
    #    has a directed cycle, and None otherwise
    def cycle(self):
        return self._cycle

    # certify that digraph is either acyclic or has a directed cycle
    def _check(self):

        # edge-weighted digraph is cyclic
        if self.has_cycle():
            # verify cycle
            first = None
            last = None
            for e in self.cycle():
                if first is None: 
                    first = e
                if last is not None:
                    if last.to_vertex() != e.from_vertex():
                        print("cycle edges {} and {} not incident".format(last, e))
                        return False    
                last = e
            
            if last.to_vertex() != first.from_vertex():
                print("cycle edges {} and {} not incident".format(last, first))
                return False
        
        return True


def main(args):
    # create random DAG with V vertices and E edges; then add F random edges
    V = int(args[0])
    E = int(args[1])
    F = int(args[2])
    G = EdgeWeightedDigraph(V)
    vertices = [i for i in range(V)]
    stdrandom.shuffle(vertices)
    for i in range(E):
        while True:
            v = stdrandom.uniformInt(0,V)
            w = stdrandom.uniformInt(0,V)
            if v >= w:
                break
        weight = stdrandom.uniformFloat(0.0, 1.0)
        G.add_edge(DirectedEdge(v, w, weight))

    # add F extra edges
    for i in range(F):
        v = stdrandom.uniformInt(0,V)
        w = stdrandom.uniformInt(0,V)
        weight = stdrandom.uniformFloat(0.0, 1.0)
        G.add_edge(DirectedEdge(v, w, weight))
    
    print(G)

    # find a directed cycle
    finder = EdgeWeightedDirectedCycle(G)
    if finder.has_cycle():
        print("Cycle: ")
        for e in finder.cycle():
            print("{}  ".format(e), end='')    
        print()
    # or give topologial sort
    else: 
        print("No directed cycle")
    

if __name__ == '__main__':
    main(sys.argv[1:])



# Created for BADS 2018
# See README.md for details
# This is python3 
import sys

from itu.algs4.searching.sequential_search_st import SequentialSearchST


class SeparateChainingHashST:
	"""
	The SeparateChainingHashST class represents a symbol table of dynamic
	key-value pairs.
	It supports the usual put, get, contains, delete, size, 
	and is-empty methods.
	It also provides a keys method for iterating over all of the keys.
	A symbol table implements the associative array abstraction:
	when associating a value with a key that is already in the symbol table,
	the convention is to replace the old value with the new value.
	Unlike the Map-class in Java, this class uses the convention that
	values cannot be null/None. Setting the
	value associated with a key to None is equivalent to deleting the key
	from the symbol table.

	This implementation uses a separate chaining hash table. It requires that
	the key type overrides the __eq__ and __hash__ methods.
	The expected time per put, contains, or remove
	operation is constant, subject to the uniform hashing assumption.
	The size, and is-empty operations take constant time.
	Construction takes constant time.
	"""
	def __init__(self, M = 997):
		self.M = M #Hash table size
		self.N = 0 #Number of pairs
		self.st = [SequentialSearchST() for i in range(0,M)] #Array of ST objects

	def _hash(self, key):
		#Hash value between 0 and M-1
		return (hash(key) & 0x7fffffff) % self.M

	def get(self, key):
		"""
		Returns the value associated with the specified key.

		:param key: the key
		:returns: the value associated with the key in the symbol table;
					None if no such value
		:raises ValueError: if key is None
		"""
		if(key is None):
			raise ValueError("argument to get() is None")
		i = self._hash(key)
		return self.st[i].get(key)

	def put(self, key, value):
		"""
		Inserts the specified key-value paur into the symbol table, overwriting the old
		value with the new value if the symbol table already contains the specified key.
		Deletes the specified key (and its associated value) from this symbol table
		if the specified value is None.

		:param key: the key
		:param value: the value
		:raises ValueError: if key is None.
		"""
		if(key is None):
			raise ValueError("first argument to put() is None")
		if(value is None):
			self.delete(key)
			return
		i = self._hash(key)
		if(not self.st[i].contains(key)):
			self.N += 1
		self.st[i].put(key, value)
		self.st[self._hash(key)].put(key, value)

	def size(self):
		"""
		Returns the number of key-value pairs in this symbol table.

		:returns: the number of key-value pairs in this symbol table.
		"""
		return self.N

	def is_empty(self):
		"""
		Returns true if the symbol table is empty.

		:returns: True if this symbol table is empty;
					False otherwise.
		"""
		return (self.N == 0)

	def contains(self, key):
		"""
		Returns true if this symbol table contains the specified key.

		:param key: the key
		:returns: True if this symbol table contains the key;
					False otherwise.
		:raises ValueError: if key is None.
		"""
		if(key == None):
			raise ValueError("argument to contains() is None")
		return self.get(key) is not None

	def delete(self, key):
		"""
		Removes the specified key and its associated value from this symbol table
		(if the key is in this symbol table).

		:param key: the key
		:raises ValueError: if key is None
		"""
		if(key is None):
			raise ValueError("argument to delete() is None")
		i = self._hash(key)
		if(self.st[i].contains(key)):
			self.N -= 1
		self.st[i].delete(key)

	def keys(self):
		"""
		Returns the keys in the symbol table as an iterable
		:returns: A list containing all keys
		"""
		keys = []
		for i in range(0,self.M):
			for key in self.st[i].keys():
				keys.append(key)
		return keys

	def __len__(self):
		return self.size()
def main():
	"""
	Unit tests the SeparateChainingHashST data type.
	"""
	st = SeparateChainingHashST()
	i = 0
	for key in sys.argv[1:]:
		st.put(key, i)
		i += 1
	for key in st.keys():
		print('{} {}'.format(key, st.get(key)))
		
if __name__ == "__main__":
	main()



# Created for BADS 2018
# See README.md for details
# Python 3


"""
This module implements a data type for solving the all-pairs shortest 
paths problem in edge-weighted digraphs where the edge weights are nonnegative.
"""

import sys

from itu.algs4.graphs.dijkstra_sp import DijkstraSP
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.stdlib import instream


class DijkstraAllPairsSP:
    """
    This implementation runs Dijkstra's algorithm from each vertex. The constructor 
    takes time proportional to V (E log V) and uses space proprtional to V2, where 
    V is the number of vertices and E is the number of edges. Afterwards, the dist() 
    and hasPath() methods take constant time and the path() method takes time 
    proportional to the number of edges in the shortest path returned.

    For additional documentation, see Section 4.4 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """
    
    def __init__(self, edge_weighted_digraph):
        """
        Computes a shortest paths tree from each vertex to to every other vertex in the edge-weighted digraph G.
        
        :param edge_weighted_digraph: the edge-weighted digraph
        """
        self._all = []
        for v in range(edge_weighted_digraph.V()):
            self._all.append(DijkstraSP(edge_weighted_digraph, v))
        
    def path(self, source, target):
        """
        Returns a shortest path from source vertex to the target vertex.
        
        :param source: the source vertex
        :param target: the destination vertex
        
        :returns: a shortest path from the source vertex to the target vertex as an iterable of edges, 
                  and None if no such path
        """
        self._validateVertex(source)
        self._validateVertex(target)
        
        return self._all[source].path_to(target)
    
    def has_path(self, source, target):
        """
        Is there a path from the source vertex to the target vertex?
        
        :param source: the source vertex
        :param target: the target vertex
        
        :returns: True if there is a path from the source to the target, and False otherwise
        """
        return self.dist(source, target) < float('inf')
    
    def dist(self, source, target):
        """
        Returns the length of a shortest path from the source vertex to the target vertex.
        
        :param source: the source vertex
        :param target: the target vertex
        
        :returns: the length of a shortest path from the source vertex to the target vertex; 
                  float('inf') if no such path
        """
        self._validateVertex(source)
        self._validateVertex(target)   
        
        return self._all[source].dist_to(target)
    
    # throw a ValueError unless 0 <= v < V
    def _validateVertex(self, v):
        V = len(self._all)
        if v < 0 or v >= V:
            raise ValueError('vertex {} is not between 0 and {}'.format(v, (V-1)))


if __name__ == '__main__':
    # Create stream from file or the standard input,
    # depending on whether a file name was passed.
    stream = sys.argv[1] if len(sys.argv) > 1 else None
    
    # Create a DijkstraAllPairsSP data structure
    g = EdgeWeightedDigraph.from_stream(instream.InStream(stream))
    dijkstra_all = DijkstraAllPairsSP(g)
    
    # Print the shortest path distances between all possible pairs of vertices.
    for source in range(g.V()):
        for target in range(g.V()):
            print(dijkstra_all.dist(source, target))



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
stdrandom.py

The stdrandom module defines functions related to pseudo-random
numbers.
"""

#-----------------------------------------------------------------------

import math
import random

#-----------------------------------------------------------------------

def seed(i=None):
    """
    Seed the random number generator as hash(i), where i is an int.
    If i is None, then seed using the current time or, quoting the
    help page for random.seed(), "an operating system specific
    randomness source if available."
    """
    random.seed(i)

#-----------------------------------------------------------------------

def uniform(hi):
    """
    Return an integer chosen uniformly from the range [0, hi).
    """
    return random.randrange(0, hi)
    
#-----------------------------------------------------------------------
def uniformInt(lo, hi):
    """
    Return an integer chosen uniformly from the range [lo, hi).
    """
    return random.randrange(lo, hi)
    
#-----------------------------------------------------------------------

def uniformFloat(lo, hi):
    """
    Return a number chosen uniformly from the range [lo, hi).
    """
    return random.uniform(lo, hi)

#-----------------------------------------------------------------------

def bernoulli(p=0.5):
    """
    Return True with probability p.
    """
    return random.random() < p

#-----------------------------------------------------------------------

def binomial(n, p=0.5):
    """
    Return the number of heads in n coin flips, each of which is
    heads with probability p.
    """
    heads = 0
    for i in range(n):
        if bernoulli(p):
            heads += 1
    return heads
    
#-----------------------------------------------------------------------

def gaussian(mean=0.0, stddev=1.0):
    """
    Return a float according to a standard Gaussian distribution
    with the given mean (mean) and standard deviation (stddev).
    """

    # Approach 1:
    # return random.gauss(mu, sigma)

    # Approach 2: Use the polar form of the Box-Muller transform.
    x = uniformFloat(-1.0, 1.0)
    y = uniformFloat(-1.0, 1.0)
    r = x*x + y*y
    while (r >= 1) or (r == 0):
        x = uniformFloat(-1.0, 1.0)
        y = uniformFloat(-1.0, 1.0)
        r = x*x + y*y
    g = x * math.sqrt(-2 * math.log(r) / r)
    # Remark:  x * math.sqrt(-2 * math.log(r) / r)
    # is an independent random gaussian
    return mean + stddev * g

#-----------------------------------------------------------------------

def discrete(a):
    """
    Return a float from a discrete distribution: i with probability
    a[i].  Precondition: the elements of array a sum to 1.
    """
    r = uniformFloat(0.0, sum(a))
    subtotal = 0.0
    for i in range(len(a)):
        subtotal += a[i]
        if subtotal > r:
            return i
    #return len(a) - 1

#-----------------------------------------------------------------------

def shuffle(a):
    """
    Shuffle array a.
    """
    
    # Approach 1:
    # for i in range(len(a)):
    #     j = i + uniformInt(len(a) - i)
    #     temp = a[i]
    #     a[i] = a[j]
    #     a[j] = temp
    
    # Approach 2:
    random.shuffle(a)

#-----------------------------------------------------------------------

def exp(lambd):
    """
    Return a float from an exponential distribution with rate lambd.
    """
    
    # Approach 1:
    # return random.expovariate(lambd)
    
    # Approach 2:
    return -math.log(1 - random.random()) / lambd
    
#-----------------------------------------------------------------------

def _main():
    """
    For testing.
    """
    import sys
    from itu.algs4.stdlib import stdio
    seed(1)
    n = int(sys.argv[1])
    for i in range(n):
        stdio.writef(' %2d '   , uniformInt(10, 100))
        stdio.writef('%8.5f '  , uniformFloat(10.0, 99.0))
        stdio.writef('%5s '    , bernoulli())
        stdio.writef('%5s '    , binomial(100, .5))
        stdio.writef('%7.5f '  , gaussian(9.0, .2))
        stdio.writef('%2d '    , discrete([.5, .3, .1, .1]))
        stdio.writeln()

if __name__ == '__main__':
    _main()
    
#-----------------------------------------------------------------------

# python stdrandom.py 5
#  27 60.65914 False    41 9.01682  0
#  55 46.88378  True    48 8.90171  0 
#  58 92.96468  True    52 9.12770  0 
#  79 64.41387 False    47 9.49241  0 
#  29 32.30299  True    45 8.77630  1 



# Created for BADS 2018
# See README.md for details
# Python 3

from itu.algs4.errors.errors import IllegalArgumentException, NoSuchElementException


class IndexMinPQ:
    """
    The IndexMinPQ class represents an indexed priority queue of generic keys.
    It supports the usual insert and delete-the-minimum
    operations, along with delete and change-the-key
    methods. In order to let the client refer to the keys on the priority queue,
    an integer between 0 and maxN - 1
    is associated with each key-the client uses this integer to specify
    which key to delete or change.
    It also supports methods for peeking at the minimum key,
    testing if the priority queue is empty, and iterating through
    the keys.
    This implementation uses a binary heap along with an array to associate
    keys with integers, in the given range.
    The insert, delete-the-minimum, delete, change-key, decrease-key, and increase-key
    operations take logarithmic time.
    The is-empty, size, min-index, min-key, and key-of operations take constant time.
    Construction takes time proportional to the specified capacity.
    """
    def __init__(self, max_n):
        """
        Initializes an empty indexed priority queue with indices between 0
        and max_n - 1.
        :param max_n: the keys on this priority queue are indices from 0 to max_n - 1
        :raises IllegalArgumentException: if max_n < 0
        """
        self.max_n = max_n
        self.n = 0
        self.keys = [None] * (max_n + 1)
        self.pq = [0] * (max_n + 1)
        self.qp = [-1] * (max_n + 1)

    def insert(self, i, key):
        """
        Associates key with index i
        :param i: an index
        :param key: the key to associate with index i
        :raises IllegalArgumentException: unless 0 <= i < max_n
        :raises IllegalArgumentException: if there already is an item associated with index i
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is not within range")
        if self.contains(i):
            raise IllegalArgumentException("index is already in the priority queue")
        self.n += 1
        self.qp[i] = self.n
        self.pq[self.n] = i
        self.keys[i] = key
        self._swim(self.n)

    def contains(self, i):
        """
        Is i an index on this priority queue?
        :param i: an index
        :return: True if i is an index on this priority queue False otherwise
        :rtype: bool
        :raises IllegalArgumentException: unless 0 <= i < max_n
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is not within range")
        return self.qp[i] != -1

    def change_key(self, i, key):
        """
        Change the key associated with index i to the specified value.
        :param i: the index of the key to change
        :param key: change the key associated with index i to this key
        :raises IllegalArgumentException: unless 0 <= i < max_n
        :raises NoSuchElementException: if no key is associated with index i
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is not within range")
        if not self.contains(i):
            raise NoSuchElementException("index is not in the priority queue")
        self.keys[i] = key
        self._swim(self.qp[i])
        self._sink(self.qp[i])

    def decrease_key(self, i, key):
        """
        Decrease the key associated with index i to the specified value.
        :param i: the index of the key to decrease
        :param key: decrease the key associated with index i to this key
        :raises IllegalArgumentException: unless 0 <= i < max_n
        :raises IllegalArgumentException: if key >= key_of(i)
        :raises NoSuchElementException: if no key is associated with index i
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is not within range")
        if not self.contains(i):
            raise IllegalArgumentException("index is not in the priority queue")
        if self.keys[i] <= key:
            raise IllegalArgumentException("calling decrease_key() with given argument would not strictly decrease the key")
        self.keys[i] = key
        self._swim(self.qp[i])

    def increase_key(self, i, key):
        """
        Increase the key associated with index i to the specified value.
        :param i: the index of the key to increase
        :param key: increase the key associated with index i to this key
        :raises IllegalArgumentException: unless 0 <= i < max_n
        :raises IllegalArgumentException: if key <= key_of(i)
        :raises NoSuchElementException: if no key is associated with index i
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is not within range")
        if not self.contains(i):
            raise NoSuchElementException("index is not in the priority queue")
        if self.keys[i] >= key:
            raise IllegalArgumentException("calling increase_key() with given argument would not strictly increase the key")
        self.keys[i] = key
        self._sink(self.qp[i])

    def delete(self, i):
        """
        Remove the key associated with index i
        :param i: the index of the key to remove
        :raises IllegalArgumentException: unless 0 <= i < max_n
        :raises NoSuchElementException: if no key is associated with index i
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is not in range")
        if not self.contains(i):
            raise NoSuchElementException("index is not in the priority queue")
        index = self.qp[i]
        self._exch(index, self.n)
        self.n -= 1
        self._sink(index)
        self.keys[i] = None
        self.qp[i] = -1

    def min_index(self):
        """
        Returns an index associated with a minimum key.
        :return: an index associated with a minimum key
        :rtype: int
        :raises NoSuchElementException: if this priority queue is empty
        """
        if self.n == 0:
            raise NoSuchElementException("Priority queue underflow")
        return self.pq[1]

    def min_key(self):
        """
        Returns a minimum key.
        :return: a minimum key
        :raises NoSuchElementException: if this priority queue is empty
        """
        if self.n == 0:
            raise NoSuchElementException("Priority queue underflow")
        return self.keys[self.pq[1]]

    def del_min(self):
        """
        Removes a minimum key and returns its associated index.
        :return: an index associated with a minimum key
        :raises NoSuchElementException: if this priority queue is empty
        :rtype: int
        """
        if self.n == 0:
            raise NoSuchElementException("Priority queue underflow")
        _min = self.pq[1]
        self._exch(1, self.n)
        self.n -= 1
        self._sink(1)
        self.qp[_min] = -1
        self.keys[_min] = None
        self.pq[self.n+1] = -1
        return _min

    def is_empty(self):
        """
        Returns True if this priority queue is empty.
        :return: True if this priority queue is empty False otherwise
        :rtype: bool
        """
        return self.n == 0

    def size(self):
        """
        Returns the number of keys on this priority queue.
        :return: the number of keys on this priority queue
        :rtype: int
        """
        return self.n

    def __len__(self):
        return self.size()

    def key_of(self, i):
        """
        Returns the key associated with index i.
        :param i: the index of the key to return
        :return: the key associated with index i
        :raises IllegalArgumentException: unless 0 <= i < max_n
        :raises NoSuchElementException: if no key is associated with index i
        """
        if i < 0 or i >= self.max_n:
            raise IllegalArgumentException("index is out of range")
        if not self.contains(i):
            raise IllegalArgumentException("index is not on the priority queue")
        return self.keys[i]

    def _exch(self, i, j):
        """
        Exchanges the position of items at index i and j on the heap.
        :param i: index of the first item
        :param j: index of the second item
        """
        self.pq[i], self.pq[j] = self.pq[j], self.pq[i]
        self.qp[self.pq[i]] = i
        self.qp[self.pq[j]] = j

    def _greater(self, i, j):
        """
        Returns True if key at index i on the heap is greater than key at index j.
        :param i: index of the first item
        :param j: index of the second item
        :return: True if key at index i on the heap is greater than key at index j
        :rtype: bool
        """
        return self.keys[self.pq[i]] > self.keys[self.pq[j]]

    def _swim(self, k):
        """
        Moves item at index k up to a legal position on the heap.
        :param k: Index of the item on the heap to be moved
        """
        while k > 1 and self._greater(k//2, k):
            self._exch(k, k//2)
            k = k//2

    def _sink(self, k):
        """
        Moves item at index k down to a legal position on the heap.
        :param k: Index of the item on the heap to be moved
        """
        while 2*k <= self.n:
            j = 2*k
            if j < self.n and self._greater(j, j+1):
                j += 1
            if not self._greater(k, j):
                break
            self._exch(k, j)
            k = j

    def __iter__(self):
        """
        Iterates over all the items in this priority queue in ascending order.
        """
        copy = IndexMinPQ(len(self.pq)-1)
        for i in range(1, self.n + 1):
            copy.insert(self.pq[i], self.keys[self.pq[i]])
        while not copy.is_empty():
            yield copy.del_min()


def main():
    """
    Inserts a bunch of strings to an indexed priority queue,
    deletes and prints them, inserts them again, and prints them
    using an iterator.
    """
    strings = ["it", "was", "the", "best", "of", "times", "it", "was", "the", "worst"]
    pq = IndexMinPQ(len(strings))
    for i in range(len(strings)):
        pq.insert(i, strings[i])
    while not pq.is_empty():
        i = pq.del_min()
        print("{} {}".format(i, strings[i]))
    print()
    for i in range(len(strings)):
        pq.insert(i, strings[i])
    for i in pq:
        print("{} {}".format(i, strings[i]))
    while not pq.is_empty():
        pq.del_min()


if __name__ == '__main__':
    main()



# Created for BADS 2018
# See README.md for details
# Python 3
import sys

from itu.algs4.errors.errors import (
    IllegalArgumentException,
    NoSuchElementException,
    UnsupportedOperationException,
)


"""
Set implementation using Python's set() type.
Does not allow duplicates.
"""

class SET:
    # Initializes a new set that is an independent copy of the specified set, or an empty one.
    def __init__(self, x=None):
        self._set = set() if x is None else x._set.copy()

    # Adds the key to this set (if it is not already present).
    def add(self, key):
        if key is None:
            raise IllegalArgumentException("called add() with a None key")
        self._set.add(key)

    # Returns true if this set contains the given key.
    def contains(self, key):
        if key is None:
            raise IllegalArgumentException("called contains() with a None key")
        return key in self._set

    # Removes the specified key from this set (if the set contains the specified key).
    def delete(self, key):
        if key is None:
            raise IllegalArgumentException("called delete() with a None key")
        if self.contains(key):
            self._set.remove(key)

    # Returns the number of keys in this set.
    def size(self):
        return len(self._set)
        
    def __len__(self):
        return self.size()

    # Returns true if this set is empty.
    def is_empty(self):
        return self.size() == 0
    # Returns all of the keys in this set, as an iterator.
    # To iterate over all of the keys in a set named set, use the
    # foreach notation: for key in set.
    def __iter__(self):
        for k in self._set:
            yield k

    # Returns the largest key in this set.
    def max(self):
        if self.is_empty():
            raise NoSuchElementException("called max() with empty set")
        return max(self._set)

    # Returns the smallest key in this set.
    def min(self):
        if self.is_empty():
            raise NoSuchElementException("called min() with empty set")
        return min(self._set)

    # Returns the smallest key in this set greater than or equal to key.
    def ceiling(self, key):
        if key is None:
            raise IllegalArgumentException("called ceiling() with None key")
        ceiling = None
        for k in self:
            if (ceiling is None and k >= key) or (ceiling is not None and k>=key and k<ceiling):
                ceiling = k
        if ceiling is None:
            raise NoSuchElementException("all keys are less than " + str(key))
        return ceiling

    # Returns the largest key in this set less than or equal to key.
    def floor(self, key):
        if key is None:
            raise IllegalArgumentException("called floor() with None key")
        floor = None
        for k in self:
            if (floor is None and k <= key) or (floor is not None and k<=key and k>floor):
                floor = k
        if floor is None:
            raise NoSuchElementException("all keys are greater than " + str(key))
        return floor

    # Returns the union of this set and that set.
    def union(self, that):
        if that is None:
            raise IllegalArgumentException("called union() with a None argument")
        c = set()
        for x in self:
            c.add(x)
        for x in that:
            c.add(x)
        return c

    # Returns the intersection of this set and that set.
    def intersects(self, that):
        if that is None:
            raise IllegalArgumentException("called intersects() with a null argument")
        c = set()
        if self.size() < that.size():
            for x in self:
                if that.contains(x):
                    c.add(x)
        else:
            for x in that:
                if self.contains(x):
                    c.add(x)
        return c
     
    # Compares this set to the specified set.
    def __eq__(self, other):
        if other is None:
            return False
        if type(other) != type(self):
            return False
        return self._set == other._set

    # This operation is not supported because sets are mutable.
    def hashCode(self):
        raise UnsupportedOperationException("hashCode() is not supported because sets are mutable")
    
    # Returns a string representation of this set.
    def __repr__(self):
        s = []
        for item in self:
            s.append("{}".format(item))
        return "{ " + ''.join(s) + " }"


def main():
    set = SET()
    print("set = " + str(set))

    # insert some keys
    set.add("www.cs.princeton.edu")
    set.add("www.cs.princeton.edu")    # overwrite old value
    set.add("www.princeton.edu")
    set.add("www.math.princeton.edu")
    set.add("www.yale.edu")
    set.add("www.amazon.com")
    set.add("www.simpsons.com")
    set.add("www.stanford.edu")
    set.add("www.google.com")
    set.add("www.ibm.com")
    set.add("www.apple.com")
    set.add("www.slashdot.com")
    set.add("www.whitehouse.gov")
    set.add("www.espn.com")
    set.add("www.snopes.com")
    set.add("www.movies.com")
    set.add("www.cnn.com")
    set.add("www.iitb.ac.in")


    print(set.contains("www.cs.princeton.edu"))
    print(not set.contains("www.harvardsucks.com"))
    print(set.contains("www.simpsons.com"))
    print()

    print("ceiling(www.simpsonr.com) = " + set.ceiling("www.simpsonr.com"))
    print("ceiling(www.simpsons.com) = " + set.ceiling("www.simpsons.com"))
    print("ceiling(www.simpsont.com) = " + set.ceiling("www.simpsont.com"))
    print("floor(www.simpsonr.com)   = " + set.floor("www.simpsonr.com"))
    print("floor(www.simpsons.com)   = " + set.floor("www.simpsons.com"))
    print("floor(www.simpsont.com)   = " + set.floor("www.simpsont.com"))
    print()

    print("set = " + str(set))
    print()

    # print out all keys in this set in lexicographic order
    for s in set:
        print(s)

    print()
    set2 = SET(set)
    print(set == set2)

if __name__ == '__main__':
    main()



# Created for BADS 2018
# see README.md for details
# Python 3

import sys
from typing import Generic, Iterator, List, Optional, TypeVar

from itu.algs4.errors.errors import NoSuchElementException
from itu.algs4.stdlib import stdio

Key = TypeVar('Key')

class MaxPQ(Generic[Key]):
    """
    The MaxPQ class represents a priority queue of generic keys.
    It supports the usual insert and delete-the-maximum
    operations, along with methods for peeking at the maximum key,
    testing if the priority queue is empty, and iterating through the keys.
    This implementation uses a binary heap.
    The insert and delete-the-maximum operations take logarithmic amortized time.
    The max, size and is_empty operations take constant time.
    Construction takes time proportional to the specified capacity.
    """
    def __init__(self, _max: int = 1):
        """
        Initializes an empty priority queue with the given initial capacity
        :param _max: the initial capacity, default value is 1
        """
        self._pq: List[Optional[Key]] = [None] * (_max + 1)
        self._n = 0

    def insert(self, x: Key) -> None:
        """
        Adds a new key to this priority queue.
        :param x: the new key to add to this priority queue
        """
        if self._n == len(self._pq) - 1:
            self._resize(2 * len(self._pq))
        self._n += 1
        self._pq[self._n] = x
        self._swim(self._n)

    def max(self) -> Key:
        """
        Returns a largest key on this priority queue.
        :return: a largest key on the priority queue
        :raises NoSuchElementException: if this priority queue is empty
        """
        if self.is_empty():
            raise NoSuchElementException("Priority queue underflow")

        assert self._pq[1] is not None
        return self._pq[1]

    def del_max(self) -> Key:
        """
        Removes and returns a largest key on this priority queue.
        :return: a largest key on this priority queue
        :raises NoSuchElementException: if this priority queue is empty
        """
        if self.is_empty():
            raise NoSuchElementException("Priority queue underflow")

        _max = self._pq[1]
        assert _max is not None
        self._exch(1, self._n)
        self._n -= 1
        self._sink(1)
        self._pq[self._n + 1] = None
        if self._n > 0 and self._n == (len(self._pq) - 1) // 4:
            self._resize(len(self._pq) // 2)
        return _max

    def is_empty(self) -> bool:
        """
        Returns True if this priority queue is empty.
        :return: True if this priority queue is empty otherwise False
        :rtype: bool
        """
        return self._n == 0

    def size(self) -> int:
        """
        Returns the number of keys on this priority queue.
        :return: the number of keys on this priority queue
        :rtype: int
        """
        return self._n

    def __len__(self) -> int:
        return self.size()

    def _sink(self, k) -> None:
        """
        Moves item at index k down to a legal position on the heap.
        :param k: Index of the item to be moved
        """
        while 2*k <= self._n:
            j = 2*k
            if j < self._n and self._less(j, j+1):
                j += 1
            if not self._less(k, j):
                break
            self._exch(k, j)
            k = j

    def _swim(self, k: int) -> None:
        """
        Moves item at index k up to a legal position on the heap.
        :param k: Index of the item to be moved
        """
        while k > 1 and self._less(k//2, k):
            self._exch(k, k//2)
            k = k//2

    def _resize(self, capacity: int):
        """
        Copies the contents of the heap to a new array of size capacity.
        :param capacity: The capacity of the new array
        """
        temp: List[Optional[Key]] = [None] * capacity
        for i in range(1, self._n + 1):
            temp[i] = self._pq[i]
        self._pq = temp

    def _less(self, i: int, j: int):
        """
        Check if item at index i is less than item at index j on the heap.
        :param i: index of the first item
        :param j: index of the second item
        :return: True if item at index i is smaller than item at index j otherwise False
        """
        return self._pq[i] < self._pq[j]

    def _exch(self, i: int, j: int):
        """
        Exchanges the position of items at index i and j on the heap.
        :param i: index of the first item
        :param j: index of the second item
        """
        self._pq[i], self._pq[j] = self._pq[j], self._pq[i]

    def __iter__(self) -> Iterator[Key]:
        """
        Iterates over all the items in this priority queue in heap order.
        """
        copy: MaxPQ[Key] = MaxPQ(self.size())
        for i in range(1, self._n + 1):
            key = self._pq[i]
            assert key is not None
            copy.insert(key)
        for i in range(1, copy._n + 1):
            yield copy.del_max()


def main():
    """
    Reads strings from stdin and adds them to a priority queue.
    When reading a '-' it removes a maximum item on the priority queue and prints it to stdout.
    Prints the amount of items left on the priority queue
    """
    pq = MaxPQ()
    while not stdio.isEmpty():
        item = stdio.readString()
        if item != '-':
            pq.insert(item)
        elif not pq.is_empty():
            print(pq.del_max())
    print("({} left on pq)".format(pq.size()))


if __name__ == '__main__':
    main()



# Created for BADS 2018
# see README.md for details
# This is python3 

from itu.algs4.graphs.digraph import Digraph
from itu.algs4.searching.binary_search_st import BinarySearchST
from itu.algs4.stdlib import stdio
from itu.algs4.stdlib.instream import InStream


class SymbolDigraph:
    """
    The SymbolDigraph class representsclass represents a digraph, where the
    vertex names are arbitrary strings.
    By providing mappings between vertex names and integers,
    it serves as a wrapper around the Digraph data type, which assumes the 
    vertex names are integers between 0 and V - 1.
    It also supports initializing a symbol digraph from a file.
    
    This implementation uses an ST to map from strings to integers,
    an array to map from integers to strings, and a Digraph to store
    the underlying graph.
    The index_of and contains operations take time 
    proportional to log V, where V is the number of vertices.
    The name_of operation takes constant time.
    """

    def __init__(self, filename, delimiter):
        """
        Initializes a digraph from a file using the specified delimiter.
        Each line in the file contains
        the name of a vertex, followed by a list of the names
        of the vertices adjacent to that vertex, separated by the delimiter.

        :param filename: the name of the file
        :param delimiter: the delimiter between fields
        """
        self._st = BinarySearchST()             # string -> index

        # First pass builds the index by reading strings to associate
        # distinct strings with an index
        stream = InStream(filename)
        while not stream.isEmpty():
            a = stream.readLine().split(delimiter)
            for i in range(len(a)):
                if not self._st.contains(a[i]):
                    self._st.put(a[i], self._st.size())

        stdio.writef("Done reading %s\n", filename)

        # inverted index to get keys in an array
        self._keys = [None] * self._st.size()   # index  -> string
        for name in self._st.keys():
            self._keys[self._st.get(name)] = name
    
        # second pass builds the graph by connecting first vertex on each
        # line to all others
        self._graph = Digraph(self._st.size())    # the underlying graph
        stream = InStream(filename)
        while stream.hasNextLine():
            a = stream.readLine().split(delimiter)
            v = self._st.get(a[0])
            for i in range(1, len(a)):
                w = self._st.get(a[i])
                self._graph.add_edge(v, w)

    def contains(self, s):
        """
        Does the graph contain the vertex named s?

        :param s: the name of a vertex
        :return:s true if s is the name of a vertex, and false otherwise        
        """
        return self._st.contains(s)

    def index_of(self, s):
        """
        Returns the integer associated with the vertex named s.
        :param s: the name of a vertex
        :returns: the integer (between 0 and V - 1) associated with the vertex named s
        """
        return self._st.get(s)
    
    def name_of(self, v):
        """
        Returns the name of the vertex associated with the integer v.
        @param  v the integer corresponding to a vertex (between 0 and V - 1) 
        @throws IllegalArgumentException unless 0 <= v < V
        @return the name of the vertex associated with the integer v
        """
        self._validateVertex(v)
        return self._keys[v]

    def digraph(self):
        return self._graph


    def _validateVertex(self, v):
        # throw an IllegalArgumentException unless 0 <= v < V
        V = self._graph.V()
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))

if __name__ == "__main__":
    import sys

    filename  = sys.argv[1]    
    delimiter = sys.argv[2]
    sg = SymbolDigraph(filename, delimiter)
    graph = sg.digraph()
    while stdio.hasNextLine():
        source = stdio.readLine()
        if sg.contains(source):
            s = sg.index_of(source)
            for v in graph.adj(s):
                stdio.writef("\t%s\n",sg.name_of(v))
        else:
            stdio.writef("input not contain '%i'", source)



from itu.algs4.fundamentals.queue import Queue
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.digraph import Digraph


class DepthFirstOrder:
    """
    The DepthFirstOrder class represents a data type for determining depth-first 
    search ordering of the vertices in a digraph or edge-weighted digraph, including 
    preorder, postorder, and reverse postorder.
    
    This implementation uses depth-first search. The constructor takes time proportional 
    to V + E (in the worst case), where V is the number of vertices and E is the number 
    of edges. Afterwards, the preorder, postorder, and reverse postorder operation takes 
    take time proportional to V.

    For additional documentation, see Section 4.2 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """
    
    def __init__(self, digraph):
        """
        Determines a depth-first order for the digraph.
        
        :param digraph: the digraph to check
        """
        self._pre  = [0]*digraph.V()
        self._post = [0]*digraph.V()
        self._preorder  = Queue()
        self._postorder = Queue()
        self._marked = [False]*digraph.V()
        
        self._pre_counter  = 0
        self._post_counter = 0
        
        if isinstance(digraph, Digraph):
            dfs = self._dfs
        else:
            dfs = self._dfs_edge_weighted
        
        for v in range(digraph.V()):
            if (not self._marked[v]):
                dfs(digraph, v)
    
    def post(self, v=None):
        """
        Either returns the postorder number of vertex v or, if v is None, 
        returns the vertices in postorder.
        
        :param v: None, or the vertex to return the postorder number of
        :return: if v is None, the vertices in postorder, otherwise the postorder
        number of v
        """
        if v is None:
            return self._postorder
        else:
            self._validate_vertex(v)
            return self._post[v]
    
    def pre(self, v=None):
        """
        Either returns the preorder number of vertex v or, if v is None, 
        returns the vertices in preorder.
        
        :param v: None, or the vertex to return the preorder number of
        :return: if v is None, the vertices in preorder, otherwise the preorder
                number of v
        """
        if v is None:
            return self._preorder
        else:
            self._validate_vertex(v)
            return self._pre[v]    
        
    def reverse_post(self):
        """
        Returns the vertices in reverse postorder.
        
        :return: the vertices in reverse postorder, as an iterable of vertices
        """
        reverse = Stack()
        for v in self._postorder:
            reverse.push(v)
        return reverse
    
    # run DFS in digraph G from vertex v and compute preorder/postorder
    def _dfs(self, digraph, v):
        self._marked[v] = True
        self._pre[v] = self._pre_counter
        self._pre_counter += 1
        self._preorder.enqueue(v)
        for w in digraph.adj(v):
            if not self._marked[w]:
                self._dfs(digraph, w)
        self._postorder.enqueue(v)
        self._post[v] = self._post_counter
        self._post_counter += 1
    
    # run DFS in edge-weighted digraph G from vertex v and compute preorder/postorder
    def _dfs_edge_weighted(self, graph, v):
        self._marked[v] = True
        self._pre[v] = self._pre_counter
        self._pre_counter += 1
        self._preorder.enqueue(v)
        for edge in graph.adj(v):
            w = edge.to_vertex()
            if not self._marked[w]:
                self._dfs_edge_weighted(graph, w)
        self._postorder.enqueue(v)
        self._post[v] = self._post_counter
        self._post_counter += 1    
    
    # throw an IllegalArgumentException unless 0 <= v < V
    def _validate_vertex(self, v):
        V = len(self._marked)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}", v, V-1)
    
    # check that pre() and post() are consistent with pre(v) and post(v)
    def _check(self):
        # check that post(v) is consistent with post()
        r = 0
        for v in self.post():
            if self.post(v) != r:
                print("post(v) and post() inconsistent")
                return False
            r += 1

        # check that pre(v) is consistent with pre()
        r = 0
        for v in self.pre():
            if self.pre(v) != r:
                print("pre(v) and pre() inconsistent")
                return False
            r += 1

        return True



# Created for BADS 2018
# See README.md for details
# This is python3
from math import sqrt

from itu.algs4.searching.seperate_chaining_hst import SeparateChainingHashST


class SparseVector:
	"""
	The SparseVector class represents a d-dimensional mathematical vector.
	Vectors are mutable: their values can be changed after they are created.
	It includes methods for addition, subtraction,
	dot product, scalar product, unit vector and Euclidean norm.

	The implementation is a symbol table of indices and values for which the vector
	coordinates are nonzero. This makes it efficient when most of the vector coordinates
	are zero.
	"""
	def __init__(self, d):
		"""
		Initializes a d-dimensional zero vector.
		:param d: the dimension of the vector
		"""
		self.d = d
		self.st = SeparateChainingHashST()

	def put(self, i, value):
		"""
		Sets the ith coordinate of this vector to the specified value.

		:param i: the index
		:param value: the new value
		:raises ValueError: unless i is between 0 and d-1
		"""
		if(i < 0 or i >= self.d):
			raise ValueError("Illegal index")
		if(value == 0.0):
			self.st.delete(i)
		else:
			self.st.put(i,value)
	def get(self, i):
		"""
		Returns the ith coordinate of this vector.

		:param i: the index
		:returns: the value of the ith coordinate of this vector
		:raises ValueError: unless i is between 0 and d-1
		"""
		if(i < 0 or i >= self.d):
			raise ValueError("Illegal index")
		if(self.st.contains(i)):
			return self.st.get(i)
		else:
			return 0.0

	def nnz(self):
		"""
		Returns the number of nonzero entries in this vector.

		:returns: the number of nonzero entries in this vector.
		"""
		return self.st.size()

	def dimension(self):
		"""
		Returns the dimension of this vector.
		
		:returns: the dimension of this vector.
		"""
		return self.d

	def dot(self, that):
		"""
		Returns the inner product of this vector with the specified vector.

		:param that: the other vector
		:returns: the dot product between this vector and that vector
		:raises ValueError: if the lengths of the two vectors are not equal
		"""
		if(self.d != that.d):
			raise ValueError("Vector lengths disagree")
		sum = 0.0

		#iterate over the vector with the fewest nonzeroes
		if(self.st.size() <= that.st.size()):
			for i in self.st.keys():
				if(that.st.contains(i)):
					sum += self.get(i) * that.get(i)
		else:
			for i in that.st.keys():
				if(self.st.contains(i)):
					sum += self.get(i) * that.get(i)
		return sum

	def magnitude(self):
		"""
		Returns the magnitude of this vector.
		This is also known as the L2 norm or the Euclidean norm.

		:returns: the magnitude of this vector
		"""
		return sqrt(self.dot(self))

	def scale(self, alpha):
		"""
		Returns the scalar-vector product of this vector with the specified scalar.

		:param alpha: the scalar
		:returns: the scalar-vector product of this vector with the specified scalar
		"""
		c = SparseVector(self.d)
		for i in self.st.keys():
			c.put(i, alpha*self.get(i))
		return c

	def plus(self, that):
		"""
		Returns the sum of this vector and the specified vector.

		:param that: the vector to add to this vector
		:returns: the sum of this vector and that vector
		:raises ValueError: if the dimension of the two vectors are not equal
		"""
		if(self.d != that.d):
			raise ValueError("Vector lengths disagree")
		c = SparseVector(self.d)
		for i in self.st.keys():
			c.put(i, self.get(i))
		for i in that.st.keys():
			c.put(i, that.get(i) + c.get(i))
		return c

	def __repr__(self):
		return ''.join( ("(%s,%s)" % (str(i), self.st.get(i))) for i in self.st.keys())
		
def main():
	"""
	Unit tests the SparseVector data type.
	"""
	a = SparseVector(10)
	b = SparseVector(10)
	a.put(3, 0.50)
	a.put(9, 0.75)
	a.put(6, 0.11)
	a.put(6, 0.00)
	b.put(3, 0.60)
	b.put(4, 0.90)
	print("a       = {}".format(a))
	print("b       = {}".format(b))
	print("a dot b = {}".format(a.dot(b)))
	print("a + b   = {}".format(a.plus(b)))
if __name__ == "__main__":
    main()
